#!/usr/local/bin/perl -w
#
# Start up a bunch of solservers. One for each port in the list.
#
# WARNING: To run as a chron job you must run this from a .csh wrapper
#          script because crontab environment does not inherit user variables.
#
# The startsolserver script does the following:
#  - check if process is already running, noop if it is, else start it
#  - if host_port combo is new, create a dir and copy needed files
#
# Note chrontasks run in ~/ so full path is needed in all scripts

use strict;
use Sys::Hostname;

my $rootdir = "$ENV{TPP_HOME}/solserver";    # probably "/app/postproc"

my $hostlong = hostname();
my @parts = split ('\.',$hostlong);  # insure we have the short hostname
my $host = $parts[0];
my $instFile = "hypinst.";
my $date = localtime();

print "--------------------------------------------------------\n";
print "Checking/restarting solservers on $host on $date\n";

# Read portlist from cfg file
open (PORTS, "${rootdir}/bin/portlist.cfg") 
   || die "No ${rootdir}/bin/portlist.cfg file.\n";
my @portlist = <PORTS>;

my $progStr = "";

# loop thru the ports
foreach my $port (@portlist) {

  chomp $port;
  next unless ($port =~ /^\d+$/);  # skip non-numbers

  $progStr = "${rootdir}/bin/solserver.pl -p $port";

  print "Checking: $progStr\n";

  if (&procExists($progStr)) {
    print "Already running.\n";
  } else {

    &startInstance($port, $progStr);
    $date = localtime();
    print "Started instance on port $port on $date.\n";
  }

}

# Refresh the common station list
# 3/22/07 - needs csh wrapper to run as cron job on linux
#:: Turned off for NC; we get our files by a different method
#:: print "Refreshing common station list.\n";
#:: print `${rootdir}/bin/refreshstalist.pl`;

#
# ===================================================================
#
sub procExists ($) {

   my ($string) = @_;

   my $res = `ps -ef | grep "${string}" | grep -v "grep" `;

   return ($res ne "");
   
}
#
sub startInstance ($) {

   my ($port, $progStr) = @_;

   my $ssdir = "${host}_${port}";

   chdir $rootdir or die "Can't chdir to $rootdir.\n $!";

  #create the working dir if it doesn't already exist
   if (! -e $ssdir) {

     print "Making $ssdir\n";
     mkdir ($ssdir) or die "Can't make dir $ssdir\n $!";
     chdir $ssdir or die "chdir to $ssdir failed\n $!";
     mkdir ("data") or die "Can't make dir data\n $!";
     symlink ("$rootdir/hypinstFiles/$instFile", "$instFile") 
	 or die "Can't make symbolic link \'$instFile\': $!\n";

   } else {

     chdir $ssdir or die "chdir failed\n $!";

   }

# handle log file 
   my $logfile = "solserver.log";  #current

   # rename previous with current time stamp (could use stat for creation time)
   if (-e $logfile) {
     my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime();
     my $stamp = sprintf "%4d%02d%02d%02d%02d",$year+1900,$mon+1,$mday,$hour,$min;
     rename ($logfile, "solserver${stamp}.log");
   }
  print `$progStr > $logfile 2>&1 &`;

 }

